PImage canvas;
int playerX, playerY;
int pixelSize = 5;
int worldSize = 128;
int blastRadius = 6;
boolean dead;
int bombCount;
int delay;
boolean bombKeyPressed;
int[][] wall;
Bomb[] bomb;
Monster[] monster;
boolean[] keyState;

void setup() {
  size(worldSize * pixelSize, worldSize * pixelSize, P2D);
  canvas = new PImage(worldSize * pixelSize, worldSize * pixelSize);
  wall = new int[worldSize][worldSize];
  for (int y = 0; y < worldSize; y ++) {
    for (int x = 0; x < worldSize; x ++) {
      wall[x][y] = 255;
    }
  }
  bomb = new Bomb[16];
  monster = new Monster[16];
  
  int roomWidth = 24;
  int roomHeight = 24;
  for (int iY = 0; iY < 4; iY ++) {
    for (int iX = 0; iX < 4; iX ++) {
      if (((iX == 0) && (iY == 0)) || (random(1) < 0.666)) {
        int variation = (int) random(2);
        MakeRoom((roomWidth + 6) * iX + 4 + 2 - variation, 
          4 + (roomHeight + 6) * iY + 2 - variation, 
          roomWidth + 2 * variation, roomHeight + 2 * variation);
        if ((iX != 0) && (iY != 0)) {
          if (random(1) < 0.6) {
            PlaceMonsters(2 + (int) random(8), (roomWidth + 6) * iX + 4 + 2 - variation, 
              4 + (roomHeight + 6) * iY + 2 - variation, 
              roomWidth + 2 * variation, roomHeight + 2 * variation);
          }
          else if (random(1) > 0.0) {
            PlaceBombs((int) random(4), roomWidth / 2 + (roomWidth + 6) * iX + 4 + 2 - variation, 
              roomHeight / 2 + 4 + (roomHeight + 6) * iY + 2 - variation);
          }
        }
      }
    }
  }
  playerX = 16;
  playerY = 16;
  dead = false;
  bombCount = 10;

  keyState = new boolean[256];
  for (int i = 0; i < 256; i ++) {
    keyState[i] = false;
  }
  delay = 0;
  bombKeyPressed = true;
}

void draw() {
  background(128, 128, 128);
  noStroke();
  for (int y = 0; y < 128; y ++) {
    for (int x = 0; x < 128; x ++) {
      if (wall[x][y] > 0) {
        fill(0, 0, 0);
        rect(x * pixelSize, y * pixelSize, pixelSize - 1, pixelSize - 1);
      }
      else if (wall[x][y] < 0) {
        fill(255, 255, 255);
        rect(x * pixelSize, y * pixelSize, pixelSize - 1, pixelSize - 1);
      }
    }
  }
  
  // Draw Player
  fill(255, 255, 255);   
  if (!dead) {
    rect(pixelSize * playerX, pixelSize * playerY, pixelSize - 1, pixelSize - 1);
  }
  
  // Bombs
  for (int i = 0; i < bomb.length; i ++) {
    if (bomb[i] != null) {
      bomb[i].Update();
      bomb[i].Draw();
      if (bomb[i].active == false) {
        bomb[i] = null;
      }
    }
  }

  // Monsters
  for (int i = 0; i < monster.length; i ++) {
    if (monster[i] != null) {
      monster[i].Update();
      monster[i].Draw();
      if (monster[i].active == false) {
        monster[i] = null;
      }
    }
  }
  int count = 0;
  for (int i = 0; i < monster.length; i ++) {
    if (monster[i] != null) {
      count ++;
    }
  }
  if (count <= 0) {
    setup();
  }
  
  // Bomb Count
  for (int i = 0; i < bombCount; i ++) {
    DrawPixel(worldSize - 2 - 2 * i, worldSize - 2, 255, 255, 255);
  }
  
  delay ++;
  if (delay > 3) {
    delay = 0;
    int dX = 0;
    int dY = 0;
    if (keyState[37] == true) {
      print("left pressed\n");
      dX -= 1;
    }
    if (keyState[38] == true) {
      print("up pressed\n");
      dY -= 1;
    }
    if (keyState[39] == true) {
      print("right pressed\n");
      dX += 1;
    }
    if (keyState[40] == true) {
      print("down pressed\n");
      dY += 1;
    }
    if (keyState[32] == true) {
      if (bombKeyPressed == false) {
        PlaceBomb(playerX, playerY);
        bombKeyPressed = true;
      }
    }
    else if (keyState[32] == false) {
      bombKeyPressed = false;
    }
    if (dead == false) {
      if (wall[playerX + dX][playerY + dY] <= 0) {
        if (wall[playerX + dX][playerY + dY] == -1) {
          wall[playerX + dX][playerY + dY] = 0;
          bombCount += 1;
        }
        playerX += dX;
        playerY += dY;
      }
    }
  }
}

void keyPressed() {
  keyState[keyCode] = true;
  if (key == 'r') {
    setup();
  }
}

void keyReleased() {
  keyState[keyCode] = false;
}

void MakeRoom(int left, int top, int width, int height) {
  print("Making room\n");
  for (int y = top; y < top + height; y ++) {
    for (int x = left; x < left + width; x ++) {
      wall[x][y] = 0;
    }
  }
}

void DrawPixel(int x, int y, int r, int g, int b) {
  fill(r, g, b);   
  rect(pixelSize * x, pixelSize * y, pixelSize - 1, pixelSize - 1);
}

void PlaceBomb(int x, int y) {
  if ((dead == false) && (bombCount > 0)) {
    bombCount --;
    int i = 0;
    boolean checking = true;
    while (checking) {
      if (bomb[i] == null) {
        bomb[i] = new Bomb(x, y);
        checking = false;  
      }
      else {
        i ++;
        if (i >= bomb.length) {
          checking = false;
        }
      }
    } 
  }
}

void PlaceBombs(int count, int x, int y) {
  int i = 0;
  for (int iX = 0; iX < 2; iX ++) {
    for (int iY = 0; iY < 2; iY ++) {
      if (i < count) {
        wall[x + 4 * iX][y + 4 * iY] = -1;
        i ++;
      }
    }
  }
}

void PlaceMonsters(int count, int left, int top, int width, int height) {
  for (int i = 0; i < count; i ++) {
    int targetX = (int) (left + random(width));
    int targetY = (int) (top + random(height));
    Monster newMonster = new Monster(targetX, targetY);
    int i2 = 0;
    boolean checking = true;
    while (checking) {
      if (monster[i2] == null) {
        monster[i2] = newMonster;
        checking = false;
      }
      else {
        i2 ++;
        if (i2 >= monster.length - 1) {
          checking = false;
        }
      }
    }
  }
}


//hi sparky
